// hlavickove subory *******************************************************
// systemove funkcie
#include <stdio.h>
#include <math.h>
// GL a GLU kniznica
#include <GL/gl.h>
#include <GL/glu.h>
// funkcie SDL kniznice
#include <SDL/SDL.h>

// .h subor
#include "mOGL.h"

#define PI    3.141592653589
#define PI180 0.017453292519

GLUquadricObj *qo; // quadratic objekt

// konstruktor *************************************************************
mOGL::mOGL()
{
 Cnum=0;

 // rozmery okna
 ViewPort[0]=0; ViewPort[1]=0; ViewPort[2]=0; ViewPort[3]=0;

 StartInfoPos=10.0f; // pozicia vypisu 1. textu pocas startu aplikacie

 t_a=0; t_b=0; t_c=0; // pomocne premenne

 lighting_onoff=1;  // osvetlenie
 help_onoff=0;      // napoveda
 scene_draw_type=0; // standardny pohlad na scenu
 pause_motion=0;    // casovo zavisly pohyb

 scene_roty=0.0f;   // rotacia sceny

 wave_time=0;       // rychlost vlnenia sa zastavy
}

// destruktor **************************************************************
mOGL::~mOGL()
{

}

// vypis info pocas startu aplikacie ***************************************
void mOGL::PrintStartInfo(char *infotext)
{
 ogl_Text.OpenDrawString();
 ogl_Text.DrawString(10.0f,StartInfoPos,infotext);
 ogl_Text.CloseDrawString();

 // vymena zobrazovacich zasobnikov, zobrazenie sceny
 SDL_GL_SwapBuffers();

 // dalsi text bude vypisany na novu poziciu
 StartInfoPos+=20.0f;
 // pauza aby bol text aspon chvilku viditelny
 SDL_Delay(400);
}

// inicializacia OpenGL - nastavenie parametrov pre tvorbu OpenGL sceny ****
bool mOGL::gl_initialize(void)
{
 // nastavenie 8-mich vseobecnych materialov
 for (t_a=0;t_a<8;t_a++) {
  ogl_Material[t_a].amb[0]=0.3f; ogl_Material[t_a].amb[1]=0.3f;
  ogl_Material[t_a].amb[2]=0.3f; ogl_Material[t_a].amb[3]=1.0f;
  ogl_Material[t_a].dif[0]=0.6f; ogl_Material[t_a].dif[1]=0.6f;
  ogl_Material[t_a].dif[2]=0.6f; ogl_Material[t_a].dif[3]=1.0f;
  ogl_Material[t_a].spe[0]=1.0f; ogl_Material[t_a].spe[1]=1.0f;
  ogl_Material[t_a].spe[2]=1.0f; ogl_Material[t_a].spe[3]=1.0f;
  ogl_Material[t_a].amb_dif[0]=0.25f; ogl_Material[t_a].amb_dif[1]=0.25f;
  ogl_Material[t_a].amb_dif[2]=0.25f; ogl_Material[t_a].amb_dif[3]=1.0f;
  ogl_Material[t_a].emi[0]=0.0f; ogl_Material[t_a].emi[1]=0.0f;
  ogl_Material[t_a].emi[2]=0.0f; ogl_Material[t_a].emi[3]=1.0f;
  ogl_Material[t_a].shi=0.0f;
  ogl_Material[t_a].transp=0.0f;
  ogl_Material[t_a].sillum[0]=0.0f; ogl_Material[t_a].sillum[1]=0.0f;
  ogl_Material[t_a].sillum[2]=0.0f; ogl_Material[t_a].sillum[3]=1.0f;
 }
 // nastavenie 8-mich vseobecnych svetiel
 for (t_a=0;t_a<8;t_a++) {
  ogl_Light[t_a].amb[0]=0.25f; ogl_Light[t_a].amb[1]=0.25f;
  ogl_Light[t_a].amb[2]=0.25f; ogl_Light[t_a].amb[3]=1.0f;
  ogl_Light[t_a].dif[0]=0.5f; ogl_Light[t_a].dif[1]=0.5f;
  ogl_Light[t_a].dif[2]=0.5f; ogl_Light[t_a].dif[3]=1.0f;
  ogl_Light[t_a].spe[0]=0.75f; ogl_Light[t_a].spe[1]=0.75f;
  ogl_Light[t_a].spe[2]=0.75f; ogl_Light[t_a].spe[3]=1.0f;
  ogl_Light[t_a].pos[0]=-2.0f; ogl_Light[t_a].pos[1]=4.0f;
  ogl_Light[t_a].pos[2]=4.0f; ogl_Light[t_a].pos[3]=1.0f;
  // pos[3]=1.0f - vsesmerove svetlo, 0.0f - smerove svetlo
  ogl_Light[t_a].spot[0]=0.0f;
  ogl_Light[t_a].spot[1]=0.0f;
  ogl_Light[t_a].spot[2]=-1.0f;
 }

 // STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ --

 // nastavenie OpenGL zasobnika farieb a hlbky *****************************
 glClearColor(0.2f,0.2f,0.2f,0.5f); // zasobnik farieb bude vymazavany sivou farbou
 glClearDepth(1.0f);                // zasobnik hlbky bude vymazavany nastavenim na 1.0f
 glClearStencil(0);                 // stencil zasobnik bude vymazavany nastavenim na 0

 glDepthFunc(GL_LEQUAL);  // typ porovnavacej funkcie pre urcenie Z-vzdialenosti
 glEnable(GL_DEPTH_TEST); // zapnutie testovania hlbky

 glDisable(GL_CULL_FACE); // nastavenie poradia, v ktorom budu definovane vrcholy objektu
 glFrontFace(GL_CCW);     // reprezentovat privratene resp. odvratene plochy,
 glCullFace(GL_BACK);     // CCW - proti smeru hod. ruciciek, CW - v smere hod. ruciciek
 glEnable(GL_CULL_FACE);  // odvratene plochy sa nebudu spracovavat

 // nastavenie parametrov 1. OpenGL svetla
 glLightfv(GL_LIGHT0,GL_AMBIENT,ogl_Light[0].amb);
 glLightfv(GL_LIGHT0,GL_DIFFUSE,ogl_Light[0].dif);
 glLightfv(GL_LIGHT0,GL_SPECULAR,ogl_Light[0].spe);
 glLightfv(GL_LIGHT0,GL_POSITION,ogl_Light[0].pos);
 glEnable(GL_LIGHT0); // ON
 // nastavenie parametrov 2. OpenGL svetla
 glLightfv(GL_LIGHT1,GL_AMBIENT,ogl_Light[1].amb);
 glLightfv(GL_LIGHT1,GL_DIFFUSE,ogl_Light[1].dif);
 glLightfv(GL_LIGHT1,GL_SPECULAR,ogl_Light[1].spe);
 glLightfv(GL_LIGHT1,GL_POSITION,ogl_Light[1].pos);
 //glEnable(GL_LIGHT1); // OFF
 if (lighting_onoff) glEnable(GL_LIGHTING);
 else glDisable(GL_LIGHTING);
 // svetlo sa pocita len pre privratene strany
 glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,0);
 // svetelne vypocty su lokalne
 glLightModelf(GL_LIGHT_MODEL_LOCAL_VIEWER,GL_TRUE);

 // vseobecny material objektov
 glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,ogl_Material[0].amb_dif);
 // farba bude mat vplyv iba na 'diffuse' komponent materialu
 glColorMaterial(GL_FRONT,GL_DIFFUSE);
 // pre Gourandove svetelne vypocty bude pouzity material priradeny objektu,
 // ktory bude ale stale ovplyvneny farbou zadavanou prikazom glColor
 glEnable(GL_COLOR_MATERIAL);
 // zapnutie gourand-jemneho tienovania
 glShadeModel(GL_SMOOTH);
 // trojuholniky budu vyplnene
 glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);

 // realnejsie perspektivne vypocty a korekcie
 glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);

 qo=gluNewQuadric();              // pre kvadraticke objekty
 gluQuadricNormals(qo,GL_SMOOTH); // zapnutie generovania normalovych vektorov
 gluQuadricTexture(qo,GL_TRUE);   // zapnutie generovania texturovacich suradnic

 // nastavenie modu generovania koordinatov S
 glTexGeni(GL_S,GL_TEXTURE_GEN_MODE,GL_OBJECT_LINEAR);
 // nastavenie modu generovania koordinatov T
 glTexGeni(GL_T,GL_TEXTURE_GEN_MODE,GL_OBJECT_LINEAR);

 // ZAPNUTIE TEXTUROVANIA
 glEnable(GL_TEXTURE_2D);

 // INICIALIZACIA SCENY INICIALIZACIA SCENY INICIALIZACIA SCENY ------------

 // vymazanie zasobnika farieb, hlbky a stencil zasobnika
 glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

 // inicializacia vypisovania OpenGL textu
 if (ogl_Text.TextInitialize()) {
  fprintf(stderr,"Chyba inicializacie fontu!\n"); return(1);
 }
 // potvrdenie startu OpenGL
 PrintStartInfo("Nastavenie OpenGL stavoveho stroja...");
 PrintStartInfo("OK");

 // zameriavaci kriz
 PrintStartInfo("Nahratie textury zameriavacieho kriza...");
 if (ogl_Texture.LoadCrosshairTexture("data/gfx/crosshair0.bmp")) {
  fprintf(stderr,"Chyba nahratia textury zameriavacieho kriza!\n"); return(1);
 }
 PrintStartInfo("OK");

 // zastava
 PrintStartInfo("Priprava zastavy...");
 // nahratie textur
 if (ogl_Texture.LoadTexture(0,"data/sk.tga",TEXTURE_CLAMP|TEXTURE_MIPMAP|TEXTURE_TRILINEAR)) {
  fprintf(stderr,"Chyba nahratia textury pre zastavu!\n"); return(1);
 }
 if (ogl_Texture.LoadTexture(1,"data/marble.bmp",TEXTURE_MIPMAP|TEXTURE_TRILINEAR)) {
  fprintf(stderr,"Chyba nahratia textury pre podstavec!\n"); return(1);
 }
 if (ogl_Texture.LoadTexture(2,"data/metal.bmp",TEXTURE_MIPMAP|TEXTURE_TRILINEAR)) {
  fprintf(stderr,"Chyba nahratia textury pre zrd!\n"); return(1);
 }

 // pociatocny vypocet vrcholov siete tvoriacej zastavu
 for (t_a=0;t_a<33;t_a++) {
  for (t_b=0;t_b<32;t_b++) {
   flag_v[t_a][t_b][0]=(GLfloat) t_a/8.0f;
   flag_v[t_a][t_b][1]=(GLfloat) t_b/12.0f+(GLfloat) cos((GLfloat) t_a*1.0f*11.0f*PI180)/12.0f;
   flag_v[t_a][t_b][2]=(GLfloat) sin((GLfloat) t_a*1.0f*11.0f*PI180)/4.0f; // 360.0f/33=10.9090f
  }
 }
 // pociatocny vypocet normal v jednotlivych vrcholoch siete tvoriacej zastavu
 for (t_a=0;t_a<32;t_a++) {
  for (t_b=0;t_b<31;t_b++) {
   // 1. vektor
   v11=flag_v[t_a+1][t_b][0]-flag_v[t_a][t_b][0];
   v12=flag_v[t_a+1][t_b][1]-flag_v[t_a][t_b][1];
   v13=flag_v[t_a+1][t_b][2]-flag_v[t_a][t_b][2];
   // 2. vektor
   v21=flag_v[t_a][t_b+1][0]-flag_v[t_a][t_b][0];
   v22=flag_v[t_a][t_b+1][1]-flag_v[t_a][t_b][1];
   v23=flag_v[t_a][t_b+1][2]-flag_v[t_a][t_b][2];
   // vektorovy sucin = normala
   flag_n[t_a][t_b][0]=v12*v23-v13*v22;
   flag_n[t_a][t_b][1]=v13*v21-v11*v23;
   flag_n[t_a][t_b][2]=v11*v22-v12*v21;
   // ziskanie velkosti vektora
   p1=sqrt(flag_n[t_a][t_b][0]*flag_n[t_a][t_b][0]+
           flag_n[t_a][t_b][1]*flag_n[t_a][t_b][1]+
           flag_n[t_a][t_b][2]*flag_n[t_a][t_b][2]);
   // normalizacia vektora
   if (p1!=0.0f) { flag_n[t_a][t_b][0]=flag_n[t_a][t_b][0]/p1;
                   flag_n[t_a][t_b][1]=flag_n[t_a][t_b][1]/p1;
                   flag_n[t_a][t_b][2]=flag_n[t_a][t_b][2]/p1;
   }
  }
 }
 PrintStartInfo("OK");

 // koniec pociatocnych nastaveni, zaciatok interakcie pozorovatela
 PrintStartInfo("START");

 // nastavenie farby a velkosti neskor vypisovaneho textu
 ogl_Text.SetTextColor(1.0f,1.0f,1.0f);
 ogl_Text.SetTextSize(10);

 // nastavenie startovacej farby - biela
 glColor4f(1.0f,1.0f,1.0f,1.0f);

 return(0);
}

// vykreslenie OpenGL sceny ************************************************
int mOGL::gl_draw(void)
{
 // vymazanie zasobnika farieb, hlbky a stencil zasobnika
 glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT|GL_STENCIL_BUFFER_BIT);

 // zresetovanie modelovej matice
 glLoadIdentity();

 // TRANSFORMACIA SCENY PODLA NASTAVENI AKTUALNEJ KAMERY!
 if (scene_draw_type==0) glMultMatrixf(ogl_Camera[Cnum].matrix);
 else {
  glRotatef(45.0f,1.0f,0.0f,0.0f);  // zapnutie specialneho pohladu na scenu
  glRotatef(-30.0f,0.0f,1.0f,0.0f);
 }
 // TRANSFORMACIA SCENY PODLA NASTAVENI AKTUALNEJ KAMERY!
 if (scene_draw_type==0) glTranslatef(ogl_Camera[Cnum].posx,ogl_Camera[Cnum].posy,ogl_Camera[Cnum].posz);
 else glTranslatef(-7.5f,-10.0f,-15.0f); // v pripade specialneho pohladu na scenu

 // transformacia polohy svetla podla polohy kamery
 //ogl_Light[0].pos[0]=-ogl_Camera[0].posx;
 //ogl_Light[0].pos[1]=-ogl_Camera[0].posy;
 //ogl_Light[0].pos[2]=-ogl_Camera[0].posz;

 // umiestenenie svetla na preddefinovanu poziciu
 glLightfv(GL_LIGHT0,GL_POSITION,ogl_Light[0].pos);

 // rotacia sceny
 glRotatef(30.0f,1.0f,0.0f,0.0f);
 glRotatef(scene_roty,0.0f,1.0f,0.0f);

 // priprava stencil zasobnika --------------------------------------------
 // zastava a zrd sa budu odrazat na podstavci

 // zakazanie vykreslovania pixelov do zasobnika farieb
 glColorMask(0,0,0,0);
 // zapnutie stencil testovania
 glEnable(GL_STENCIL_TEST);
 // pre vsetky pixely, ktore nie su cierne sa v zasobniku stencil nastavi 1
 glStencilFunc(GL_ALWAYS,1,1);
 glStencilOp(GL_KEEP,GL_KEEP,GL_REPLACE);

 glPushAttrib(GL_LIGHTING_BIT|GL_DEPTH_BUFFER_BIT|GL_TEXTURE_BIT);
 glDisable(GL_LIGHTING);   // vypnutie osvetlenia
 glDisable(GL_DEPTH_TEST); // vypnutie testu hlbky
 glDisable(GL_TEXTURE_2D); // vypnutie texturovania

 glPushMatrix();
 glRotatef(90.0f,1.0f,0.0f,0.0f);
 // podstavec sa vykresli do stencil zasobnika
 gluCylinder(qo,0.0f,4.0f,0.0f,32,32);
 gluCylinder(qo,4.0f,6.0f,0.4f,32,32);
 glPopMatrix();

 glPopAttrib();

 // povolenie vykreslovania pixelov do zasobnika farieb
 glColorMask(1,1,1,1);
 // do zasobnika farieb sa vykreslia len pixely, ktore prejdu stencil testom
 glStencilFunc(GL_EQUAL,1,1);
 glStencilOp(GL_KEEP,GL_KEEP,GL_KEEP);

 // zrkadlove vykreslenie zastavy a zrde ----------------------------------

 // textura pre zastavu
 glBindTexture(GL_TEXTURE_2D,ogl_Texture.texture_id[0]);

 glPushMatrix();
 // zrkadlove vykreslenie zastavy
 glScalef(1.0f,-1.0f,1.0f);
 glTranslatef(0.3f,7.3f,0.0f);

 // vykreslia sa privratene aj odvratene steny
 glDisable(GL_CULL_FACE);

 glBegin(GL_QUADS);
 t_a=0;
 // vykresli sa 1. vertikalny pas zastavy "pripojeny k zrdi"
 for (t_b=0;t_b<31;t_b++) {
  t_x =(GLfloat) t_a/32.0f;     t_y =(GLfloat) t_b/41.4f;
  t_xx=(GLfloat) (t_a+1)/32.0f; t_yy=(GLfloat) (t_b+1)/41.4f;

  glTexCoord2f(t_x,t_y); glNormal3f(0.0f,0.0f,1.0f);
  glVertex3f(-0.25f,flag_v[t_a][t_b][1],flag_v[t_a][t_b][2]/5.0f);
  glTexCoord2f(t_xx,t_y); glNormal3f(flag_n[t_a][t_b][0],flag_n[t_a][t_b][1],flag_n[t_a][t_b][2]);
  glVertex3f(flag_v[t_a][t_b][0],flag_v[t_a][t_b][1],flag_v[t_a][t_b][2]);
  glTexCoord2f(t_xx,t_yy); glNormal3f(flag_n[t_a][t_b+1][0],flag_n[t_a][t_b+1][1],flag_n[t_a][t_b+1][2]);
  glVertex3f(flag_v[t_a][t_b+1][0],flag_v[t_a][t_b+1][1],flag_v[t_a][t_b+1][2]);
  glTexCoord2f(t_x,t_yy); glNormal3f(0.0f,0.0f,1.0f);
  glVertex3f(-0.25f,flag_v[t_a][t_b+1][1],flag_v[t_a][t_b+1][2]/5.0f);
 }
 // vykresli sa zvysok zastavy
 for (t_a=0;t_a<31;t_a++) {
  for (t_b=0;t_b<31;t_b++) {
   t_x =(GLfloat) (t_a+1)/32.0f; t_y =(GLfloat) t_b/41.4f;
   t_xx=(GLfloat) (t_a+2)/32.0f; t_yy=(GLfloat) (t_b+1)/41.4f;

   glTexCoord2f(t_x,t_y); glNormal3f(flag_n[t_a][t_b][0],flag_n[t_a][t_b][1],flag_n[t_a][t_b][2]);
   glVertex3f(flag_v[t_a][t_b][0],flag_v[t_a][t_b][1],flag_v[t_a][t_b][2]);
   glTexCoord2f(t_xx,t_y); glNormal3f(flag_n[t_a+1][t_b][0],flag_n[t_a+1][t_b][1],flag_n[t_a+1][t_b][2]);
   glVertex3f(flag_v[t_a+1][t_b][0],flag_v[t_a+1][t_b][1],flag_v[t_a+1][t_b][2]);
   glTexCoord2f(t_xx,t_yy); glNormal3f(flag_n[t_a+1][t_b+1][0],flag_n[t_a+1][t_b+1][1],flag_n[t_a+1][t_b+1][2]);
   glVertex3f(flag_v[t_a+1][t_b+1][0],flag_v[t_a+1][t_b+1][1],flag_v[t_a+1][t_b+1][2]);
   glTexCoord2f(t_x,t_yy); glNormal3f(flag_n[t_a][t_b+1][0],flag_n[t_a][t_b+1][1],flag_n[t_a][t_b+1][2]);
   glVertex3f(flag_v[t_a][t_b+1][0],flag_v[t_a][t_b+1][1],flag_v[t_a][t_b+1][2]);
  }
 }
 glEnd();

 glPopMatrix();

 // textura pre zrd
 glBindTexture(GL_TEXTURE_2D,ogl_Texture.texture_id[2]);

 glPushMatrix();
 // zrkadlove vykreslenie zrde
 glScalef(1.0f,-1.0f,1.0f);
 glRotatef(90.0f,1.0f,0.0f,0.0f);
 glTranslatef(0.0f,0.0f,-2.5f); gluCylinder(qo,0.12f,0.12f,2.5f,32,32); // 4x pre opakovanie sa textury
 glTranslatef(0.0f,0.0f,-2.5f); gluCylinder(qo,0.12f,0.12f,2.5f,32,32);
 glTranslatef(0.0f,0.0f,-2.5f); gluCylinder(qo,0.12f,0.12f,2.5f,32,32);
 glTranslatef(0.0f,0.0f,-2.5f); gluCylinder(qo,0.12f,0.12f,2.5f,32,32);

 glPopMatrix();

 // vykreslia sa iba privratene steny
 glEnable(GL_CULL_FACE);

 // vypnutie stencil testovania
 glDisable(GL_STENCIL_TEST);

 // normalne vykreslenie zastavy, podstavca a zrde ------------------------

 // textura pre zastavu
 glBindTexture(GL_TEXTURE_2D,ogl_Texture.texture_id[0]);

 glPushMatrix();
 glTranslatef(0.3f,7.3f,0.0f);

 // vykreslia sa privratene aj odvratene steny
 glDisable(GL_CULL_FACE);

 glBegin(GL_QUADS);
 t_a=0;
 // vykresli sa 1. vertikalny pas zastavy "pripojeny k zrdi"
 for (t_b=0;t_b<31;t_b++) {
  t_x =(GLfloat) t_a/32.0f;     t_y =(GLfloat) t_b/41.4f;
  t_xx=(GLfloat) (t_a+1)/32.0f; t_yy=(GLfloat) (t_b+1)/41.4f;

  glTexCoord2f(t_x,t_y); glNormal3f(0.0f,0.0f,1.0f);
  glVertex3f(-0.25f,flag_v[t_a][t_b][1],flag_v[t_a][t_b][2]/5.0f);
  glTexCoord2f(t_xx,t_y); glNormal3f(flag_n[t_a][t_b][0],flag_n[t_a][t_b][1],flag_n[t_a][t_b][2]);
  glVertex3f(flag_v[t_a][t_b][0],flag_v[t_a][t_b][1],flag_v[t_a][t_b][2]);
  glTexCoord2f(t_xx,t_yy); glNormal3f(flag_n[t_a][t_b+1][0],flag_n[t_a][t_b+1][1],flag_n[t_a][t_b+1][2]);
  glVertex3f(flag_v[t_a][t_b+1][0],flag_v[t_a][t_b+1][1],flag_v[t_a][t_b+1][2]);
  glTexCoord2f(t_x,t_yy); glNormal3f(0.0f,0.0f,1.0f);
  glVertex3f(-0.25f,flag_v[t_a][t_b+1][1],flag_v[t_a][t_b+1][2]/5.0f);
 }
 // vykresli sa zvysok zastavy
 for (t_a=0;t_a<31;t_a++) {
  for (t_b=0;t_b<31;t_b++) {
   t_x =(GLfloat) (t_a+1)/32.0f; t_y =(GLfloat) t_b/41.4f;
   t_xx=(GLfloat) (t_a+2)/32.0f; t_yy=(GLfloat) (t_b+1)/41.4f;

   glTexCoord2f(t_x,t_y); glNormal3f(flag_n[t_a][t_b][0],flag_n[t_a][t_b][1],flag_n[t_a][t_b][2]);
   glVertex3f(flag_v[t_a][t_b][0],flag_v[t_a][t_b][1],flag_v[t_a][t_b][2]);
   glTexCoord2f(t_xx,t_y); glNormal3f(flag_n[t_a+1][t_b][0],flag_n[t_a+1][t_b][1],flag_n[t_a+1][t_b][2]);
   glVertex3f(flag_v[t_a+1][t_b][0],flag_v[t_a+1][t_b][1],flag_v[t_a+1][t_b][2]);
   glTexCoord2f(t_xx,t_yy); glNormal3f(flag_n[t_a+1][t_b+1][0],flag_n[t_a+1][t_b+1][1],flag_n[t_a+1][t_b+1][2]);
   glVertex3f(flag_v[t_a+1][t_b+1][0],flag_v[t_a+1][t_b+1][1],flag_v[t_a+1][t_b+1][2]);
   glTexCoord2f(t_x,t_yy); glNormal3f(flag_n[t_a][t_b+1][0],flag_n[t_a][t_b+1][1],flag_n[t_a][t_b+1][2]);
   glVertex3f(flag_v[t_a][t_b+1][0],flag_v[t_a][t_b+1][1],flag_v[t_a][t_b+1][2]);
  }
 }
 glEnd();
/*
 glDisable(GL_LIGHTING); // vykreslenie normal
 glDisable(GL_TEXTURE_2D);
 glColor4f(1.0f,0.0f,0.0f,1.0f);
 glBegin(GL_LINES);
 for (t_a=0;t_a<32;t_a++) {
  for (t_b=0;t_b<32;t_b++) {
   glVertex3f(flag_v[t_a][t_b][0],flag_v[t_a][t_b][1],flag_v[t_a][t_b][2]);
   glVertex3f(flag_v[t_a][t_b][0]+flag_n[t_a][t_b][0]/2.0f,
              flag_v[t_a][t_b][1]+flag_n[t_a][t_b][1]/2.0f,
              flag_v[t_a][t_b][2]+flag_n[t_a][t_b][2]/2.0f);
  }
 }
 glEnd();
 glColor4f(1.0f,1.0f,1.0f,1.0f);
 glEnable(GL_TEXTURE_2D);
 glEnable(GL_LIGHTING);
*/
 glPopMatrix();

 // vykreslenie zrde a podstavca
 glRotatef(90.0f,1.0f,0.0f,0.0f);

 // textura pre zrd
 glBindTexture(GL_TEXTURE_2D,ogl_Texture.texture_id[2]);
 glTranslatef(0.0f,0.0f,-2.5f); gluCylinder(qo,0.12f,0.12f,2.5f,32,32);
 glTranslatef(0.0f,0.0f,-2.5f); gluCylinder(qo,0.12f,0.12f,2.5f,32,32);
 glTranslatef(0.0f,0.0f,-2.5f); gluCylinder(qo,0.12f,0.12f,2.5f,32,32);
 glTranslatef(0.0f,0.0f,-2.5f); gluCylinder(qo,0.12f,0.12f,2.5f,32,32);

 glTranslatef(0.0f,0.0f,10.f);

 // vykreslia sa iba privratene steny
 glEnable(GL_CULL_FACE);

 glEnable(GL_TEXTURE_GEN_S); // zapnutie generovania texturovych koordinatov S
 glEnable(GL_TEXTURE_GEN_T); // zapnutie generovania texturovych koordinatov T

 // textura pre podstavec
 glBindTexture(GL_TEXTURE_2D,ogl_Texture.texture_id[1]);
 glMatrixMode(GL_TEXTURE);
 glPushMatrix();
 glLoadIdentity();
 glScalef(0.25f,0.25f,1.0f); // uprava texturovacich suradnic
 glMatrixMode(GL_MODELVIEW);

 glEnable(GL_BLEND); // zapnutie priesvitnosti
 glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
 glColor4f(1.0f,1.0f,1.0f,0.75f);
 gluCylinder(qo,0.0f,4.0f,0.0f,32,32);
 gluCylinder(qo,4.0f,6.0f,0.4f,32,32);
 glColor4f(1.0f,1.0f,1.0f,1.0f); // standardna farba
 glDisable(GL_BLEND);

 glMatrixMode(GL_TEXTURE);
 glPopMatrix(); // vratenie GL_TEXTURE matice do povodneho stavu
 glMatrixMode(GL_MODELVIEW);

 glDisable(GL_TEXTURE_GEN_S); // vypnutie automatickeho generovania koordinatov S
 glDisable(GL_TEXTURE_GEN_T); // vypnutie automatickeho generovania koordinatov T

 // vypocet novych vrcholov siete tvoriacej zastavu -----------------------

 if (wave_time==10) { // vypocet iba kazdy 10-ty snimok
  for (t_b=0;t_b<32;t_b++) {
   // ulozenie Y a Z hodnot posledneho vrchola
   p1=flag_v[31][t_b][1]; p2=flag_v[31][t_b][2];
   // ulozenie hodnot poslednej normaly
   v11=flag_n[31][t_b][0]; v12=flag_n[31][t_b][1]; v13=flag_n[31][t_b][2];

   // vzajomna vymena hodnot vrcholov a normal - posun doprava
   for (t_a=31;t_a>0;t_a--) {
    flag_v[t_a][t_b][1]=flag_v[t_a-1][t_b][1];
    flag_v[t_a][t_b][2]=flag_v[t_a-1][t_b][2];

    flag_n[t_a][t_b][0]=flag_n[t_a-1][t_b][0];
    flag_n[t_a][t_b][1]=flag_n[t_a-1][t_b][1];
    flag_n[t_a][t_b][2]=flag_n[t_a-1][t_b][2];
   }
   // zmena Y a Z hodnot prveho vrchola podla ulozenych hodnot posledneho vrchola
   flag_v[0][t_b][1]=p1; flag_v[0][t_b][2]=p2;
   // zmena hodnot prvej normaly podla ulozenych hodnot poslednej normaly
   flag_n[0][t_b][0]=v11; flag_n[0][t_b][1]=v12; flag_n[0][t_b][2]=v13;
  }
  wave_time=0;
 }
 wave_time++; // zvysenie pocitadla snimkov

 // vykreslenie zameriavacieho kriza
 ogl_Texture.DrawCrosshair(ogl_Camera[Cnum].scr_w,ogl_Camera[Cnum].scr_h);

 if (help_onoff) {
  // vypis textu - help
  ogl_Text.OpenDrawString();

  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-20.0f, "ESC ukoncenie aplikacie");

  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-60.0f, "MYS                rotacia v danom smere");
  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-80.0f, "SIPKY              posun dopredu,dozadu,vlavo,vpravo");
  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-100.0f,"SHIFT              zrychlenie posunu");

  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-140.0f,"R     reinicializacia pohladu kamery");
  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-160.0f,"M     vyplnene trojuholniky/iba hrany trojuholnikov");
  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-180.0f,"P     zapnutie/vypnutie casovo zavisleho pohybu objektov");

  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-220.0f,"H     zapnutie/vypnutie zobrazenia helpu");

  ogl_Text.CloseDrawString();
 }
 else {
  // vypis textu - poloha kamery
  ogl_Text.OpenDrawString();

  sprintf(ogl_Text.numberbuf,"x:%6.2f",-ogl_Camera[Cnum].posx);
  ogl_Text.DrawNumber(10.0f,30.0f);
  sprintf(ogl_Text.numberbuf,"y:%6.2f",-ogl_Camera[Cnum].posy);
  ogl_Text.DrawNumber(10.0f,20.0f);
  sprintf(ogl_Text.numberbuf,"z:%6.2f",-ogl_Camera[Cnum].posz);
  ogl_Text.DrawNumber(10.0f,10.0f);

  // vypis textu - pocet obrazkov za sekundu
  sprintf(ogl_Text.numberbuf,"%i fps",(GLuint) ogl_Camera[Cnum].ogl_fps);
  //sprintf(ogl_Text.numberbuf,"%f fps",ogl_Camera[Cnum].GetCameraTime()*1000.0f);
  ogl_Text.DrawNumber(10.0f,ogl_Camera[Cnum].scr_h-20.0f);

  ogl_Text.CloseDrawString();
 }

 return(0);
}

// casovo zavisla uprava objektov, resp. sceny *****************************
int mOGL::gl_iddle(void)
{
 if (pause_motion==0) {
  scene_roty+=0.04f;
 }

 return(0);
}

// zmena sposobu zobrazenia objektov ***************************************
void mOGL::change_PolygonMode(void)
{
 glGetIntegerv(GL_POLYGON_MODE,&ogl_State[0]);
 // INFO: vypis nacitaneho stavu
 //fprintf(stderr,"ogl_State=%d\n",ogl_State[0]);

 if (ogl_State[0]==GL_LINE) glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
 else glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);

 SDL_Delay(200);
}

// zap/vyp casovo zavisleho pohybu objektov ********************************
void mOGL::change_PauseMotion(void)
{
 if (pause_motion==0) pause_motion=1;
 else pause_motion=0;

 SDL_Delay(200);
}

// zap/vyp vypisu helpu ****************************************************
void mOGL::change_HelpOnOff(void)
{
 if (help_onoff==0) help_onoff=1;
 else help_onoff=0;

 SDL_Delay(200);
}
